from .types.Question import Question
from .types.Subject import Subject
import requests, json

class LoginFailure(Exception):
	pass

class QuestionFailure(Exception):
	pass

class Session:
	loggedin = False
	renew = 0
	expiration = 0
	token = ""
	subjects = None
	def __init__(self, user, passwd):
		self._login(user, passwd)
		self.subjects = self.give_subject_generator()
	def give_subject_generator(self):
		def generator(subjects, token):
			for subject in subjects:
				yield Subject(subject, token)
		resp = requests.get("https://api.wrts.nl/api/v3/subjects",headers={"x-auth-token": self.token}).json()
		return generator(resp["subjects"], self.token)
	def _login(self, user, passwd):
		resp = requests.post("https://api.wrts.nl/api/v3/auth/get_token", json={"email": user, "password": passwd}).json()
		#print(resp)
		if not resp["success"]:
			raise LoginFailure(resp["info"])
		self.loggedin = True
		self.renew = resp["renew_from"]
		self.expiration = resp["expires_at"]
		self.token = resp["auth_token"]
	def get_questions(self):
		resp = requests.get("https://api.wrts.nl/api/v3/public/qna/questions", headers={"x-auth-token": self.token}).json()
		def gen(questions, token):
			for q in questions:
				yield Question(q["id"],self.token)
		return gen(resp["results"], self.token)
	def get_question(self, id):
		return Question(id, self.token)
	def post_question(self, contents, subject, topic=None, attachments=[]):
		data = {"contents": contents, "subject_id": subject.id, "qna_attachments_attributes": attachments}
		if not topic == None:  data["topic_id"] = topic.id
		resp = requests.post("https://api.wrts.nl/api/v3/qna/questions",headers={"x-auth-token": self.token},json=json.dumps({"qna_question":data})).json()
		if "success" in resp:
			raise QuestionFailure(resp["error"])
		return self.get_question(resp["qna_question"]["id"])



import requests

class Topic:
	def __init__(self, obj, supercat, subcat):
		self.supercat = supercat
		self.subcat = subcat

		self.combined_grade = obj["combined_grade"] # why not do this client side??
		self.country = obj["country"] # i should probably make this into some class aswell
		self.draft = obj["draft"] # what does this even mean
		self.grade_nums = obj["grade_numbers"] # convert later on
		self.id = obj["id"]
		self.relations = obj["learning_relations"] # i wonder what this means aswell
		self.question_count = obj["questions_count"] # wait this is available information???
		self.stream_ids = obj["stream_ids"] # this is very weird
		self.school_tracks = obj["school_tracks"] # this is also very weird
		self.title = obj["title"]
		self.unpublished_question_count = obj["unpublished_questions_count"] # THIS IS ALSO AVAILABLE?!?!?

class Subject:
	def __init__(self, obj, token):
		self.id = obj["id"]

		self.fallback = obj["fallback"] # i wonder what this is
		self.icon = obj["icon_url"]
		self.locale = obj["locale"] # language or somethnig
		self.logo = obj["logo"] # what is the point???
		self.mutliple_methods = obj["multiple_methods"] # wtf is this supposed to mean?
		self.name = obj["name"]
		self.quick_answer = obj["quick_answer"] # what is the point? maybe its a priority thing (french has this turned out & french is a hard language)
		self.selected_book = obj["selected_book"] # gotta do some more research on this

		self.topics = []
		response = requests.get(f"https://api.wrts.nl/api/v3/public/subjects/{self.id}/topics",headers={"x-auth-token": token}).json()["results"]
		for supercat in response:
			for subcat in supercat["subcategories"]:
				for topic in subcat["topics"]:
					self.topics.append(Topic(topic,supercat,subcat))
